mode {
	dock: top;
	padding:0 4 1 2;
	text-style: bold;
}

IndeterminateProgress {
	align: center middle;
}

Input {
	padding: 0 4;
}

ListItem {
	border: solid rgb(0, 225, 255);
}

DataTable {
	border: solid rgb(0, 225, 255);
	height: 70%;
	width: 100%;
}